@@ -167,10 +167,6 @@ typedef unsigned char bool;
 # define gettext_noop(String) String
 #endif
 
-#if (defined MB_CUR_MAX && HAVE_WCTYPE_H && HAVE_ISWCTYPE) || _LIBC
-# define RE_ENABLE_I18N
-#endif
-
 #if __GNUC__ >= 3
 # define BE(expr, val) __builtin_expect (expr, val)
 #else
@@ -190,18 +186,127 @@ typedef unsigned char bool;
 #define WIDE_NEWLINE_CHAR L'\n'
 
 /* Rename to standard API for using out of glibc.  */
+#undef rpl__wint_t
+#undef rpl__wchar_t
+#undef rpl__mbsinit
+#undef rpl__wctype_t
+#undef rpl__wctype
+#undef rpl__isascii
+#undef rpl__isalnum
+#undef rpl__iscntrl
+#undef rpl__islower
+#undef rpl__isspace
+#undef rpl__isalpha
+#undef rpl__isdigit
+#undef rpl__isprint
+#undef rpl__isupper
+#undef rpl__isblank
+#undef rpl__isgraph
+#undef rpl__ispunct
+#undef rpl__isxdigit
+#undef rpl__iswlower
+#undef rpl__iswctype
+#undef rpl__btowc
+#undef rpl__mbrtowc
+#undef rpl__mbtowc
+#undef rpl__wcrtomb
+#undef rpl__towlower
+#undef rpl__mbstate_t
+#undef rpl__MB_CUR_MAX
+#undef rpl__MB_LEN_MAX
+#undef rpl__WEOF
 #ifndef _LIBC
-# undef __wctype
-# undef __iswctype
-# define __wctype wctype
-# define __iswctype iswctype
-# define __btowc btowc
-# define __mbrtowc mbrtowc
-# define __wcrtomb wcrtomb
+# ifndef _PERL_I18N
+#   define rpl__wint_t wint_t
+#   define rpl__wchar_t wchar_t
+#   define rpl__mbsinit mbsinit
+#   define rpl__wctype_t wctype_t
+#   define rpl__wctype(c) wctype(c)
+#   define rpl__isascii(c) isascii(c)
+#   define rpl__isalnum(c) isalnum(c)
+#   define rpl__iscntrl(c) iscntrl(c)
+#   define rpl__islower(c) islower(c)
+#   define rpl__isspace(c) isspace(c)
+#   define rpl__isalpha(c) isalpha(c)
+#   define rpl__isdigit(c) isdigit(c)
+#   define rpl__isprint(c) isprint(c)
+#   define rpl__isupper(c) isupper(c)
+#   define rpl__isblank(c) isblank(c)
+#   define rpl__isgraph(c) isgraph(c)
+#   define rpl__ispunct(c) ispunct(c)
+#   define rpl__isxdigit(c) isxdigit(c)
+#   define rpl__iswlower(c) iswlower(c)
+#   define rpl__iswctype(c) iswctype(c)
+#   define rpl__btowc(c) btowc(c)
+#   define rpl__mbrtowc(pwc, s, n, ps) mbrtowc(pwc, s, n, ps)
+#   define rpl__mbtowc(pwc, s, n) mbrtowc(pwc, s, n)
+#   define rpl__wcrtomb(s, wc, ps) wcrtomb(s, wc, ps)
+#   define rpl__towlower(wc) towlower(wc)
+#   define rpl__towupper(wc) towupper(wc)
+#   define rpl__mbstate_t mbstate_t 
+#   define rpl__MB_CUR_MAX MB_CUR_MAX
+#   define rpl__MB_LEN_MAX MB_LEN_MAX
+#   define rpl__WEOF WEOF
+# else
+#   define rpl__wint_t UV
+#   define rpl__wchar_t U8
+#   define rpl__mbsinit rpl_Perl_mbsinit
+#   define rpl__wctype_t rpl_Perl_wctype_t
+#   define rpl__wctype(property) rpl_Perl_wctype(aTHX_ property)
+#   define rpl__isascii(c) rpl_Perl_isascii(aTHX_ c)
+#   define rpl__isalnum(c) rpl_Perl_isalnum(aTHX_ c)
+#   define rpl__iscntrl(c) rpl_Perl_iscntrl(aTHX_ c)
+#   define rpl__islower(c) rpl_Perl_islower(aTHX_ c)
+#   define rpl__isspace(c) rpl_Perl_isspace(aTHX_ c)
+#   define rpl__isalpha(c) rpl_Perl_isalpha(aTHX_ c)
+#   define rpl__isdigit(c) rpl_Perl_isdigit(aTHX_ c)
+#   define rpl__isprint(c) rpl_Perl_isprint(aTHX_ c)
+#   define rpl__isupper(c) rpl_Perl_isupper(aTHX_ c)
+#   define rpl__isblank(c) rpl_Perl_isblank(aTHX_ c)
+#   define rpl__isgraph(c) rpl_Perl_isgraph(aTHX_ c)
+#   define rpl__ispunct(c) rpl_Perl_ispunct(aTHX_ c)
+#   define rpl__isxdigit(c) rpl_Perl_isxdigit(aTHX_ c)
+#   define rpl__iswlower(c) rpl_Perl_iswlower(aTHX_ c)
+#   define rpl__iswctype(c, t) rpl_Perl_iswctype(aTHX_ c, t)
+#   define rpl__btowc(c) rpl_Perl_btowc(aTHX_ c)
+#   define rpl__mbrtowc(pwc, s, n, ps) rpl_Perl_mbrtowc(aTHX_ pwc, s, n, ps)
+#   define rpl__mbtowc(pwc, s, n) rpl_Perl_mbrtowc(aTHX_ pwc, s, n)
+#   define rpl__wcrtomb(s, wc, ps) rpl_Perl_wcrtomb(aTHX_ s, wc, ps)
+#   define rpl__towlower(wc) rpl_Perl_towlower(aTHX_ wc)
+#   define rpl__towupper(wc) rpl_Perl_towupper(aTHX_ wc)
+#   define rpl__mbstate_t rpl_Perl_mbstate_t 
+#   define rpl__MB_CUR_MAX rpl_Perl_MB_CUR_MAX()
+#   define rpl__MB_LEN_MAX UTF8_MAXBYTES
+#   define rpl__WEOF ((UV)-1)
+typedef enum {
+  PERL_WCTYPE_ALNUM = 1,
+  PERL_WCTYPE_ALPHA,
+  PERL_WCTYPE_CNTRL,
+  PERL_WCTYPE_DIGIT,
+  PERL_WCTYPE_GRAPH,
+  PERL_WCTYPE_LOWER,
+  PERL_WCTYPE_PRINT,
+  PERL_WCTYPE_PUNCT,
+  PERL_WCTYPE_SPACE,
+  PERL_WCTYPE_UPPER,
+  PERL_WCTYPE_XDIGIT
+} rpl_Perl_wctype_t;
+typedef struct {
+  union
+  {
+    unsigned int __wch;
+    char __wchb[4];
+  } u;
+} rpl_Perl_mbstate_t;
+# endif
 # define __regfree regfree
 # define attribute_hidden
 #endif /* not _LIBC */
 
+#if (defined MB_CUR_MAX && HAVE_WCTYPE_H && HAVE_ISWCTYPE) || _LIBC
+# define RE_ENABLE_I18N
+#endif
+
 #if __GNUC__ < 3 + (__GNUC_MINOR__ < 1)
 # define __attribute__(arg)
 #endif
@@ -366,7 +471,7 @@ typedef enum
 typedef struct
 {
   /* Multibyte characters.  */
-  wchar_t *mbchars;
+  rpl__wchar_t *mbchars;
 
   /* Collating symbols.  */
 # ifdef _LIBC
@@ -383,12 +488,12 @@ typedef struct
   uint32_t *range_starts;
   uint32_t *range_ends;
 # else /* not _LIBC */
-  wchar_t *range_starts;
-  wchar_t *range_ends;
+  rpl__wchar_t *range_starts;
+  rpl__wchar_t *range_ends;
 # endif /* not _LIBC */
 
   /* Character classes. */
-  wctype_t *char_classes;
+  rpl__wctype_t *char_classes;
 
   /* If this character set is the non-matching list.  */
   unsigned int non_match : 1;
@@ -452,9 +557,9 @@ struct re_string_t
   unsigned char *mbs;
 #ifdef RE_ENABLE_I18N
   /* Store the wide character string which is corresponding to MBS.  */
-  wint_t *wcs;
+  rpl__wint_t *wcs;
   Idx *offsets;
-  mbstate_t cur_state;
+  rpl__mbstate_t cur_state;
 #endif
   /* Index in RAW_MBS.  Each character mbs[i] corresponds to
      raw_mbs[raw_mbs_idx + i].  */
@@ -526,10 +631,10 @@ static unsigned int re_string_context_at (pTHX_ const re_string_t *input, Idx id
 #define re_string_fetch_byte(pstr) \
   ((pstr)->mbs[(pstr)->cur_idx++])
 #define re_string_first_byte(pstr, idx) \
-  ((idx) == (pstr)->valid_len || (pstr)->wcs[idx] != WEOF)
+  ((idx) == (pstr)->valid_len || (pstr)->wcs[idx] != rpl__WEOF)
 #define re_string_is_single_byte_char(pstr, idx) \
-  ((pstr)->wcs[idx] != WEOF && ((pstr)->valid_len == (idx) + 1 \
-				|| (pstr)->wcs[(idx) + 1] != WEOF))
+  ((pstr)->wcs[idx] != rpl__WEOF && ((pstr)->valid_len == (idx) + 1 \
+				|| (pstr)->wcs[(idx) + 1] != rpl__WEOF))
 #define re_string_eoi(pstr) ((pstr)->stop <= (pstr)->cur_idx)
 #define re_string_cur_idx(pstr) ((pstr)->cur_idx)
 #define re_string_get_buffer(pstr) ((pstr)->mbs)
@@ -621,7 +726,7 @@ typedef struct bin_tree_storage_t bin_tree_storage_t;
 #define IS_ENDBUF_CONTEXT(c) ((c) & CONTEXT_ENDBUF)
 #define IS_ORDINARY_CONTEXT(c) ((c) == 0)
 
-#define IS_WORD_CHAR(ch) (isalnum (ch) || (ch) == '_')
+#define IS_WORD_CHAR(ch) (rpl__isalnum (ch) || (ch) == '_')
 #define IS_NEWLINE(ch) ((ch) == NEWLINE_CHAR)
 #define IS_WIDE_WORD_CHAR(ch) (iswalnum (ch) || (ch) == L'_')
 #define IS_WIDE_NEWLINE(ch) ((ch) == WIDE_NEWLINE_CHAR)
@@ -832,7 +937,7 @@ typedef struct
   {
     unsigned char ch;
     unsigned char *name;
-    wchar_t wch;
+    rpl__wchar_t wch;
   } opr;
 } bracket_elem_t;
 
@@ -854,7 +959,7 @@ bitset_clear (bitset_t set, Idx i)
 static bool
 bitset_contain (pTHX_ const bitset_t set, Idx i)
 {
-  return (set[i / BITSET_WORD_BITS] >> i % BITSET_WORD_BITS) & 1;
+  return ((set[i / BITSET_WORD_BITS] >> i % BITSET_WORD_BITS) & 1) ? true : false;
 }
 
 static void
@@ -916,20 +1021,552 @@ re_string_char_size_at (pTHX_ const re_string_t *pstr, Idx idx)
   if (pstr->mb_cur_max == 1)
     return 1;
   for (byte_idx = 1; idx + byte_idx < pstr->valid_len; ++byte_idx)
-    if (pstr->wcs[idx + byte_idx] != WEOF)
+    if (pstr->wcs[idx + byte_idx] != rpl__WEOF)
       break;
   return byte_idx;
 }
 
-static wint_t
+static rpl__wint_t
 internal_function __attribute__ ((pure, unused))
 re_string_wchar_at (pTHX_ const re_string_t *pstr, Idx idx)
 {
   if (pstr->mb_cur_max == 1)
-    return (wint_t) pstr->mbs[idx];
-  return (wint_t) pstr->wcs[idx];
+    return (rpl__wint_t) pstr->mbs[idx];
+  return (rpl__wint_t) pstr->wcs[idx];
 }
 
+#ifndef _LIBC
+#ifdef _PERL_I18N
+size_t rpl_Perl_MB_CUR_MAX() {
+  size_t rc;
+
+  rc = rpl__MB_LEN_MAX;
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_MB_CUR_MAX() ==> %d\n", (int) rc);
+#endif
+  return rc;
+}
+/* Initalize only the first element */
+static rpl_Perl_mbstate_t Perl_internal_state = { 0 };
+
+int rpl_Perl_isascii(pTHX_ UV c) {
+  int rc;
+#ifndef isASCII_LC_uvchr
+#define isASCII_LC_uvchr isASCII_LC
+#ifndef isASCII_LC
+#define isASCII_LC isASCII
+#endif
+#endif
+  rc = isASCII_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_isascii(%d) ==> %d\n", (int) c, rc);
+#endif
+  return rc;
+}
+
+int rpl_Perl_isalnum(pTHX_ UV c) {
+  int rc;
+#ifndef isALNUM_LC_uvchr
+#define isALNUM_LC_uvchr isALNUM_LC
+#ifndef isALNUM_LC
+#define isALNUM_LC isALNUM
+#endif
+#endif
+  rc = isALNUM_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_isalnum(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_iscntrl(pTHX_ UV c) {
+  int rc;
+#ifndef isCNTRL_LC_uvchr
+#define isCNTRL_LC_uvchr isCNTRL_LC
+#ifndef isCNTRL_LC
+#define isCNTRL_LC isCNTRL
+#endif
+#endif
+  rc = isCNTRL_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_iscntrl(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_islower(pTHX_ UV c) {
+  int rc;
+#ifndef isLOWER_LC_uvchr
+#define isLOWER_LC_uvchr isLOWER_LC
+#ifndef isLOWER_LC
+#define isLOWER_LC isLOWER
+#endif
+#endif
+  rc = isLOWER_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_islower(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_isspace(pTHX_ UV c) {
+  int rc;
+#ifndef isSPACE_LC_uvchr
+#define isSPACE_LC_uvchr isSPACE_LC
+#ifndef isSPACE_LC
+#define isSPACE_LC isSPACE
+#endif
+#endif
+  rc = isSPACE_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_isspace(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_isalpha(pTHX_ UV c) {
+  int rc;
+#ifndef isALPHA_LC_uvchr
+#define isALPHA_LC_uvchr isALPHA_LC
+#ifndef isALPHA_LC
+#define isALPHA_LC isALPHA
+#endif
+#endif
+  rc = isALPHA_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_isalpha(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_isdigit(pTHX_ UV c) {
+  int rc;
+#ifndef isDIGIT_LC_uvchr
+#define isDIGIT_LC_uvchr isDIGIT_LC
+#ifndef isDIGIT_LC
+#define isDIGIT_LC isDIGIT
+#endif
+#endif
+  rc = isDIGIT_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_isdigit(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_isprint(pTHX_ UV c) {
+  int rc;
+#ifndef isPRINT_LC_uvchr
+#define isPRINT_LC_uvchr isPRINT_LC
+#ifndef isPRINT_LC
+#define isPRINT_LC isPRINT
+#endif
+#endif
+  rc = isPRINT_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_isprint(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_isupper(pTHX_ UV c) {
+  int rc;
+#ifndef isUPPER_LC_uvchr
+#define isUPPER_LC_uvchr isUPPER_LC
+#ifndef isUPPER_LC
+#define isUPPER_LC isUPPER
+#endif
+#endif
+  rc = isUPPER_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_isupper(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_isblank(pTHX_ UV c) {
+  int rc;
+#ifndef isBLANK_LC_uvchr
+#define isBLANK_LC_uvchr isBLANK_LC
+#ifndef isBLANK_LC
+#define isBLANK_LC isBLANK
+#endif
+#endif
+  rc = isBLANK_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_isblank(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_isgraph(pTHX_ UV c) {
+  int rc;
+#ifndef isGRAPH_LC_uvchr
+#define isGRAPH_LC_uvchr isGRAPH_LC
+#ifndef isGRAPH_LC
+#define isGRAPH_LC isGRAPH
+#endif
+#endif
+  rc = isGRAPH_LC_uvchr(c);
+
+#ifndef DEBUG
+  fprintf(stderr, "rpl_Perl_isgraph(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_ispunct(pTHX_ UV c) {
+  int rc;
+#ifndef isPUNCT_LC_uvchr
+#define isPUNCT_LC_uvchr isPUNCT_LC
+#ifndef isPUNCT_LC
+#define isPUNCT_LC isPUNCT
+#endif
+#endif
+  rc = isPUNCT_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_ispunct(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_isxdigit(pTHX_ UV c) {
+  int rc;
+#ifndef isXDIGIT_LC_uvchr
+#define isXDIGIT_LC_uvchr isXDIGIT_LC
+#ifndef isXDIGIT_LC
+#define isXDIGIT_LC isXDIGIT
+#endif
+#endif
+  rc = isXDIGIT_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_isxdigit(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_iswlower(pTHX_ UV wc) {
+  int rc = rpl_Perl_islower(aTHX_ wc);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_iswlower(%ld) ==> %d\n", (unsigned long) wc, rc);
+#endif
+
+  return rc;
+}
+
+/* Our mb implementations are all stateless */
+size_t rpl_Perl_mbrtowc(pTHX_ U8 *restrict pwc, const char *restrict s, size_t n, void *restrict ps) {
+  STRLEN ch_len;
+  UV     ord;
+  size_t rc;
+#ifndef NDEBUG
+  void octdump(void *mem, unsigned int len);
+#endif
+
+  if (s == NULL) {
+    pwc = NULL;
+    s = "";
+    n = 1;
+  }
+
+#ifndef NDEBUG
+  octdump(s, n);
+#endif
+
+  if (n == 0) {
+    rc = (size_t)(-2);
+  }
+  else {
+    ord = utf8n_to_uvchr((U8 *) s, n, &ch_len, UTF8_CHECK_ONLY);
+    if (ord > 0 || *s == 0) {
+      if (pwc != NULL) {
+        Copy (s, pwc, ch_len, U8);
+      }
+      rc = (ord == 0) ? 0 : ch_len;
+    } else {
+      /* Invalid */
+      errno = EILSEQ;
+      /* The conversion state is undefined, says POSIX.  */
+      rc = (size_t)(-1);
+    }
+  }
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_mbrtowc ==> %d\n", (int) rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_mbtowc(pTHX_ rpl__wchar_t *restrict pwc, const char *restrict s, size_t n) {
+  int rc;
+
+  static rpl_Perl_mbstate_t state;
+  /* If s is NULL the function has to return null or not null
+     depending on the encoding having a state depending encoding or
+     not. */
+  if (s == NULL) {
+    /* No support for state dependent encodings. */
+    rc = 0;
+  }
+  else if (*s == '\0') {
+    if (pwc != NULL) {
+      *pwc = L'\0';
+    }
+    rc = 0;
+  } else {
+    rc = rpl_Perl_mbrtowc(aTHX_ pwc, s, n, &state);
+    if (rc < 0) {
+      rc = -1;
+    }
+  }
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_mbtowc(pwc, s=\"%s\", n=%d) ==> %d\n", s, (int) n, (int) rc);
+#endif
+
+  return rc;
+}
+
+rpl__wint_t rpl_Perl_btowc (pTHX_ int c) {
+  rpl__wint_t rc = rpl__WEOF;
+
+  if (c != EOF) {
+    char buf[1];
+    rpl__wchar_t wc;
+
+    buf[0] = (U8)c;
+    if (rpl_Perl_mbtowc(aTHX_ &wc, buf, 1) >= 0) {
+      rc = wc;
+    }
+  }
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_btowc(c=%d) ==> %d\n", (int) c, (int) rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_iswctype(pTHX_ rpl__wint_t wi, wctype_t wt) {
+  int rc;
+
+  if (wi == rpl__WEOF) {
+    rc = 0;
+  } else {
+    switch (wt) {
+    case PERL_WCTYPE_ALNUM:
+      rc = isALNUM_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_ALPHA:
+      rc = isALPHA_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_CNTRL:
+      rc = isCNTRL_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_DIGIT:
+      rc = isDIGIT_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_GRAPH:
+      rc = isGRAPH_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_LOWER:
+      rc = isLOWER_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_PRINT:
+      rc = isPRINT_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_PUNCT:
+      rc = isPUNCT_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_SPACE:
+      rc = isSPACE_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_UPPER:
+      rc = isUPPER_uni((UV) wi);
+      break;
+    case PERL_WCTYPE_XDIGIT:
+      rc = isXDIGIT_uni((UV) wi);
+      break;
+    default:
+      rc = 0;
+    }
+  }
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_iswctype(wi=%ld, wt=%d) ==> %d\n", (unsigned long) wi, (int) wt, (int) rc);
+#endif
+
+  return rc;
+}
+
+rpl_Perl_wctype_t rpl_Perl_wctype(pTHX_ const char * property) {
+  rpl_Perl_wctype_t rc;
+
+  if (strncmp(property, "alnum", sizeof("alnum") - 1) == 0) {
+    rc = PERL_WCTYPE_ALNUM;
+  }
+  else if  (strncmp(property, "alpha", sizeof("alpha") - 1) == 0) {
+    rc = PERL_WCTYPE_ALPHA;
+  }
+  else if  (strncmp(property, "cntrl", sizeof("cntrl") - 1) == 0) {
+    rc = PERL_WCTYPE_CNTRL;
+  }
+  else if  (strncmp(property, "digit", sizeof("digit") - 1) == 0) {
+    rc = PERL_WCTYPE_DIGIT;
+  }
+  else if  (strncmp(property, "graph", sizeof("graph") - 1) == 0) {
+    rc = PERL_WCTYPE_GRAPH;
+  }
+  else if  (strncmp(property, "lower", sizeof("lower") - 1) == 0) {
+    rc = PERL_WCTYPE_LOWER;
+  }
+  else if  (strncmp(property, "print", sizeof("print") - 1) == 0) {
+    rc = PERL_WCTYPE_PRINT;
+  }
+  else if  (strncmp(property, "punct", sizeof("punct") - 1) == 0) {
+    rc = PERL_WCTYPE_PUNCT;
+  }
+  else if  (strncmp(property, "space", sizeof("space") - 1) == 0) {
+    rc = PERL_WCTYPE_SPACE;
+  }
+  else if  (strncmp(property, "upper", sizeof("upper") - 1) == 0) {
+    rc = PERL_WCTYPE_UPPER;
+  }
+  else if  (strncmp(property, "xdigit", sizeof("xdigit") - 1) == 0) {
+    rc = PERL_WCTYPE_XDIGIT;
+  }
+  else {
+    rc = 0;
+  }
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_wctype(property=%s) ==> %d\n", property, (int) rc);
+#endif
+
+  return rc;
+}
+
+int rpl_Perl_mbsinit(rpl__mbstate_t *ps) {
+  const char *pstate = (const char *)ps;
+  return (pstate == NULL) || (pstate[0] == 0);
+}
+
+int Perl_wctomb(pTHX_ char *restrict s, rpl__wchar_t wc) {
+  U8     d[UTF8_MAXBYTES+1];
+  bool   is_utf8 = 1;
+  U8    *bytes;
+  STRLEN len;
+  char *p;
+
+  if (s == NULL) {
+    return 0;
+  }
+
+  if (wc == 0) {
+    *s = '\0';
+    return 1;
+  }
+
+  len = uvchr_to_utf8(d, (UV) wc) - d;
+  bytes = bytes_from_utf8(d, &len, &is_utf8);
+  memcpy(s, bytes, len);
+
+  if (bytes != d) {
+    Safefree(bytes);
+  }
+  
+#ifndef NDEBUG
+  fprintf(stderr, "Perl_wctomb(%ld) ==> %d\n", (unsigned long) wc, (int) len);
+#endif
+
+  return len;
+}
+
+size_t rpl_Perl_wcrtomb (pTHX_ char *s, rpl__wchar_t wc, rpl__mbstate_t *ps) {
+  /* This implementation of wcrtomb on top of wctomb() supports only
+     stateless encodings.  ps must be in the initial state.  */
+  if (ps != NULL && !rpl_Perl_mbsinit ( ps))
+    {
+      errno = EINVAL;
+      return (size_t)(-1);
+    }
+
+  if (s == NULL)
+    /* We know the NUL wide character corresponds to the NUL character.  */
+    return 1;
+  else
+    {
+      int ret = Perl_wctomb (aTHX_ s, wc);
+
+      if (ret >= 0)
+        return ret;
+      else
+        {
+          errno = EILSEQ;
+          return (size_t)(-1);
+        }
+    }
+}
+
+rpl__wint_t rpl_Perl_towlower(pTHX_ rpl__wint_t wc) {
+  U8     s[UTF8_MAXBYTES_CASE+1];
+  rpl__wint_t rc;
+  STRLEN len;
+
+  rc = toLOWER_uni((UV) wc, s, &len);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_towlower(%d) ==> %d\n", (int) wc, (int) rc);
+#endif
+
+  return rc;
+
+}
+
+rpl__wint_t rpl_Perl_towupper(pTHX_ rpl__wint_t wc) {
+  U8     s[UTF8_MAXBYTES_CASE+1];
+  rpl__wint_t rc;
+  STRLEN len;
+
+  rc = toUPPER_uni((UV) wc, s, &len);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_towlower(%d) ==> %d\n", (int) wc, (int) rc);
+#endif
+
+  return rc;
+
+}
+
+#endif /* _PERL_I18N */
+#endif /* _LIBC */
+
 # ifndef NOT_IN_libc
 static int
 internal_function __attribute__ ((pure, unused))
@@ -953,6 +1590,23 @@ re_string_elem_size_at (pTHX_ const re_string_t *pstr, Idx idx)
       return p - pstr->mbs - idx;
     }
   else
+#  else
+#    ifdef _PERL_I18N
+    {
+      STRLEN len = pstr->len - idx;
+      U8* utf8 = bytes_to_utf8((U8 *)(pstr->mbs + idx), &len);
+      SV *sv_tmp = newSVpvn_utf8(utf8, len, 1);
+      I32 startOffset = 0;
+      I32 length = 1;
+
+      sv_pos_u2b(sv_tmp, &startOffset, &length);
+
+      Safefree(utf8);
+      SvREFCNT_dec(sv_tmp);
+
+      return length;
+    }
+#    endif
 #  endif /* _LIBC */
     return 1;
 }
@@ -976,4 +1630,57 @@ re_string_elem_size_at (pTHX_ const re_string_t *pstr, Idx idx)
 # define __attribute_warn_unused_result__ /* empty */
 #endif
 
+#ifndef NDEBUG
+#include <stdio.h>
+#include <ctype.h>
+ 
+#ifndef OCTDUMP_COLS
+#define OCTDUMP_COLS 8
+#endif
+ 
+void octdump(void *mem, unsigned int len)
+{
+  unsigned int i, j;
+        
+  for(i = 0; i < len + ((len % OCTDUMP_COLS) ? (OCTDUMP_COLS - len % OCTDUMP_COLS) : 0); i++)
+    {
+      /* print offset */
+      if(i % OCTDUMP_COLS == 0)
+        {
+          fprintf(stderr, "0x%06x: ", i);
+        }
+ 
+      /* print oct data */
+      if(i < len)
+        {
+          fprintf(stderr, "%03o ", 0xFF & ((char*)mem)[i]);
+        }
+      else /* end of block, just aligning for ASCII dump */
+        {
+          fprintf(stderr, "    ");
+        }
+                
+      /* print ASCII dump */
+      if(i % OCTDUMP_COLS == (OCTDUMP_COLS - 1))
+        {
+          for(j = i - (OCTDUMP_COLS - 1); j <= i; j++)
+            {
+              if(j >= len) /* end of block, not really printing */
+                {
+                  fprintf(stderr, "%c", ' ');
+                }
+              else if(isprint(((char*)mem)[j])) /* printable char */
+                {
+                  fprintf(stderr, "%c", 0xFF & ((char*)mem)[j]);        
+                }
+              else /* other char */
+                {
+                  fprintf(stderr, "%c", '.');
+                }
+            }
+          fprintf(stderr, "\n");
+        }
+    }
+}
+#endif /* NDEBUG */
 #endif /*  _REGEX_INTERNAL_H */
